(error "git.el no longer ships with git. It's recommended to
replace its use with Magit, or simply delete references to git.el
in your initialization file(s). See contrib/emacs/README in git's
sources (https://github.com/git/git/blob/master/contrib/emacs/README)
for suggested alternatives and for why this happened. Emacs's own
VC mode and Magit are viable alternatives.")
